@@ -3,7 +3,7 @@
 //
 // Code available from: https://verilator.org
 //
-// Copyright 2001-2023 by Wilson Snyder. This program is free software; you
+// Copyright 2001-2024 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -63,26 +63,18 @@ class VerilatedCovImp;
 ///         for (int i = 0; i < 10; ++i) m_cases[i] = 0;
 ///         // Insert
 ///         for (int i = 0; i < 10; ++i)
-///             VL_COVER_INSERT(&m_cases[i], "comment", "Coverage Case", "i", cvtToNumStr(i));
+///             VL_COVER_INSERT(covp, name(), &m_cases[i], "comment", "Coverage Case",
+///                             "i", cvtToNumStr(i));
 ///     }
 
-#define VL_COVER_INSERT(covcontextp, countp, ...) \
+#define VL_COVER_INSERT(covcontextp, name, countp, ...) \
     do { \
-        covcontextp->_inserti(countp); \
-        covcontextp->_insertf(__FILE__, __LINE__); \
-        covcontextp->_insertp("hier", name(), __VA_ARGS__); \
+        auto const ccontextp = covcontextp; \
+        ccontextp->_inserti(countp); \
+        ccontextp->_insertf(__FILE__, __LINE__); \
+        ccontextp->_insertp("hier", name, __VA_ARGS__); \
     } while (false)
 
-//=============================================================================
-// Convert VL_COVER_INSERT value arguments to strings, is \internal
-
-template <class T>
-std::string vlCovCvtToStr(const T& t) VL_PURE {
-    std::ostringstream os;
-    os << t;
-    return os.str();
-}
-
 //=============================================================================
 //  VerilatedCov
 /// Per-VerilatedContext coverage data class.
@@ -96,12 +88,13 @@ class VerilatedCovContext VL_NOT_FINAL : public VerilatedVirtualBase {
 
 public:
     // METHODS
-    /// Return default filename
-    static const char* defaultFilename() VL_PURE { return "coverage.dat"; }
+    /// Return default filename, may override with +verilator+coverage+file
+    std::string defaultFilename() VL_MT_SAFE;
     /// Make all data per_instance, overriding point's per_instance
     void forcePerInstance(bool flag) VL_MT_SAFE;
     /// Write all coverage data to a file
-    void write(const char* filenamep = defaultFilename()) VL_MT_SAFE;
+    void write(const std::string& filename) VL_MT_SAFE;
+    void write() VL_MT_SAFE { write(defaultFilename()); }
     /// Clear coverage points (and call delete on all items)
     void clear() VL_MT_SAFE;
     /// Clear items not matching the provided string
@@ -170,11 +163,10 @@ class VerilatedCov final {
 public:
     // METHODS
     /// Return default filename for the current thread
-    static const char* defaultFilename() VL_PURE { return VerilatedCovContext::defaultFilename(); }
+    static std::string defaultFilename() VL_MT_SAFE { return threadCovp()->defaultFilename(); }
     /// Write all coverage data to a file for the current thread
-    static void write(const char* filenamep = defaultFilename()) VL_MT_SAFE {
-        threadCovp()->write(filenamep);
-    }
+    static void write(const std::string& filename) VL_MT_SAFE { threadCovp()->write(filename); }
+    static void write() VL_MT_SAFE { write(defaultFilename()); }
     /// Clear coverage points (and call delete on all items) for the current thread
     static void clear() VL_MT_SAFE { threadCovp()->clear(); }
     /// Clear items not matching the provided string for the current thread
